local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local target = x.Target
local cooldown = x.Cooldown
local buff = x.Buff
local stagger = x.Stagger
local dot = x.Dot
local talent = x.Talent

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

a.spells["auto_attack"] = {
	ID = 6603--[[Auto Attack]],
	EnemyTargetNeeded = 1,
	Melee = 1,
	NotIfActive = 1,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC
	end,
}

a.spells["gift_of_the_ox"] = {
	ID = 115072--[[Expel Harm]],
	Check = function(z)
		return s.HealthPercent("player") < 80
	end,
	NoRangeCheck = true,
}

a.spells["dampen_harm,if=incoming_damage_1500ms&buff.fortifying_brew.down"] = {
	ID = 122278--[[Dampen Harm]],
	Check = function(z)
		return x.InCombat and buff.fortifying_brew.down_nogcd and s.IncomingDamage(1500) > x.MaxHealth * 0.4
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Red",
	Continue = true,
}

a.spells["fortifying_brew,if=incoming_damage_1500ms&(buff.dampen_harm.down|buff.diffuse_magic.down)"] = {
	ID = 115202--[[Fortifying Brew]],
	Check = function(z)
		return s.IncomingDamage(1500) > x.MaxHealth * 0.4 and buff.dampen_harm.down_nogcd
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Red",
	Continue = true,
}

a.spells["Provoke"] = {
	ID = 115546--[[Provoke]],
	EnemyTargetNeeded = true,
	Check = function(z)
		return x.ThreatPercent <= 99 and ( GetNumGroupMembers() > 1 or GetNumSubgroupMembers() > 0 ) and x.InCombat and s.UnitInfo("target") ~= "player" and x.InstanceType ~= "pvp"
	end,
	NoGCD = true,
	FlashColor = "Purple",
}

a.spells["Roll"] = {
	ID = 109132--[[Roll]],
	EnemyTargetNeeded = true,
	Check = function(z)
		return not s.MeleeDistance("target")
	end,
	NoGCD = true,
	FlashColor = "Pink",
	FlashID = {109132--[[Roll]], 115008--[[Chi Torpedo]]},
}

a.spells["keg_smash,if=spell_targets>=3"] = {
	ID = 121253--[[Keg Smash]],
	Check = function(z)
		return x.InCombat and x.ActiveEnemies >= 3
	end,
}

a.spells["chi_burst"] = {
	ID = 123986--[[Chi Burst]],
	NoRangeCheck = true,
	Check = function(z)
		return x.InCombat
	end,
}

a.spells["chi_wave"] = {
	ID = 115098--[[Chi Wave]],
	Check = function(z)
		return x.EnemyDetected
	end,
}

a.spells["keg_smash"] = {
	ID = 121253--[[Keg Smash]],
}

a.spells["blackout_strike"] = {
	ID = 205523--[[Blackout Strike]],
	Check = function(z)
		return x.InCombat
	end,
}

a.spells["breath_of_fire,if=buff.blackout_combo.down&(buff.bloodlust.down|(buff.bloodlust.up&&dot.breath_of_fire_dot.refreshable))"] = {
	ID = 115181--[[Breath of Fire]],
	Check = function(z)
		return x.InCombat and buff.blackout_combo.down and (buff.bloodlust.down or (buff.bloodlust.up and dot.breath_of_fire.refreshable))
	end,
	NoRangeCheck = true,
	Melee = true,
}

a.spells["rushing_jade_wind"] = {
	ID = 116847--[[Rushing Jade Wind]],
	Check = function(z)
		return x.InCombat and talent.rushing_jade_wind.enabled
	end,
	NoRangeCheck = true,
	NoPowerCheck = true,
	Melee = true,
}

a.spells["tiger_palm,if=buff.blackout_combo.up"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return x.InCombat and talent.blackout_combo.enabled and buff.blackout_combo.up
	end,
}

a.spells["tiger_palm,if=!talent.blackout_combo.enabled&cooldown.keg_smash.remains>gcd&(energy+(energy.regen*(cooldown.keg_smash.remains+gcd)))>=55"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return x.InCombat and not talent.blackout_combo.enabled and cooldown.keg_smash.remains > x.MaxGCD and (x.Energy + (x.EnergyRegen * (cooldown.keg_smash.remains_nogcd + x.MaxGCD))) >= 55
	end,
}

a.spells["purifying_brew,if=stagger.heavy|(stagger.moderate&cooldown.brews.charges_fractional>=cooldown.brews.max_charges-0.5&buff.ironskin_brew.remains>=buff.ironskin_brew.duration*2.5)"] = {
	ID = 119582--[[Purifying Brew]],
	Check = function(z)
		return stagger.heavy or (stagger.moderate and cooldown.brews.charges_fractional >= cooldown.brews.max_charges - 0.5 and buff.ironskin_brew.remains >= buff.ironskin_brew.duration * 2.5)
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["black_ox_brew,if=incoming_damage_1500ms&stagger.heavy&cooldown.brews.charges_fractional<=0.75"] = {
	ID = 115399--[[Black Ox Brew]],
	Check = function(z)
		return s.IncomingDamage(1500) > x.MaxHealth * 0.4 and stagger.heavy and cooldown.brews.charges_fractional <= 0.75
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Red",
	Continue = true,
}

a.spells["black_ox_brew,if=(energy+(energy.regen*cooldown.keg_smash.remains))<40&buff.blackout_combo.down&cooldown.keg_smash.up"] = {
	ID = 115399--[[Black Ox Brew]],
	Check = function(z)
		return (x.Energy + (x.EnergyRegen * cooldown.keg_smash.remains_nogcd)) < 40 and buff.blackout_combo.down_nogcd and cooldown.keg_smash.up_nogcd
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Red",
	Continue = true,
}

a.spells["invoke_niuzao_the_black_ox,if=target.time_to_die>45"] = {
	ID = 115315--[[Invoke Niuzao, the Black Ox]],
	Check = function(z)
		return x.InCombat and target.time_to_die > 45
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["ironskin_brew,if=buff.blackout_combo.down&cooldown.brews.charges_fractional>=cooldown.brews.max_charges-0.1-(1+buff.ironskin_brew.remains<=buff.ironskin_brew.duration*0.5)&buff.ironskin_brew.remains<=buff.ironskin_brew.duration*2"] = {
	ID = 115308--[[Ironskin Brew]],
	Check = function(z)
		return x.InCombat and buff.blackout_combo.down_nogcd and cooldown.brews.charges_fractional_nogcd >= cooldown.brews.max_charges - 0.1 - s.If(1 + buff.ironskin_brew.remains_nogcd <= buff.ironskin_brew.duration * 0.5, 1, 0) and buff.ironskin_brew.remains_nogcd <= buff.ironskin_brew.duration * 2
	end,
	NoRangeCheck = true,
	NoGCD = true,
	FlashColor = "Yellow",
	Continue = true,
}

a.spells["Spear Hand Strike"] = {
	ID = 116705--[[Spear Hand Strike]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoGCD = true,
	FlashColor = "Aqua",
}

a.spells["Paralysis"] = {
	ID = 115078--[[Paralysis]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	Check = function(z)
		return not s.Boss("target")
	end,
	FlashColor = "Aqua",
}

a.spells["Leg Sweep as Interrupt"] = {
	ID = 119381--[[Leg Sweep]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	Melee = true,
	Check = function(z)
		return not s.Boss("target")
	end,
	FlashColor = "Aqua",
}

