local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local prev_gcd = x.PrevGCD
local target = x.Target
local cooldown = x.Cooldown
local buff = x.Buff
local debuff = x.Debuff
local dot = x.Dot
local talent = x.Talent
local set_bonus = x.SetBonus
local artifact = x.Artifact
local equipped = x.Equipped

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

-- Example flash color coding:
-- White - Default
-- Yellow - Limited Time or No Global Cooldown
-- Purple - AOE or Positional Damage
-- Blue - AOE Debuff
-- Orange - Finishing Move
-- Aqua - Spell Interrupt, Reflect or Dispel
-- Green - Self Buff or Turn Autocast On
-- Red - Emergency Mitigation Cooldowns or Turn Autocast Off
-- Pink - Optional


--[[
   ____            _     _                         
  / ___|___   ___ | | __| | _____      ___ __  ___ 
 | |   / _ \ / _ \| |/ _` |/ _ \ \ /\ / / '_ \/ __|
 | |__| (_) | (_) | | (_| | (_) \ V  V /| | | \__ \
  \____\___/ \___/|_|\__,_|\___/ \_/\_/ |_| |_|___/
                                                   ]]

a.spells["invoke_xuen_the_white_tiger"] = {
	ID = 123904--[[Invoke Xuen, the White Tiger]],
	Check = function(z)
		return x.UseLongCD
	end,
	FlashColor = "Pink",
}

a.spells["blood_fury"] = {
	ID = 20572--[[Blood Fury]],
	NoGCD = true,
	CheckFirst = function(z)
		return x.UseLongCD
	end,
	FlashColor = "Yellow",
}

a.spells["berserking"] = {
	ID = 26297--[[Berserking]],
	NoGCD = true,
	CheckFirst = function(z)
		return x.UseLongCD
	end,
	FlashColor = "Yellow",
}

a.spells["arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5"] = {
	ID = 25046--[[Arcane Torrent]],
	NoGCD = true,
	CheckFirst = function(z)
		return x.UseLongCD and x.MaxChi - x.Chi >= 1 and (x.MaxEnergy - s.Power("player", Enum.PowerType.Energy )) / x.EnergyRegen >= 0.5
	end,
	FlashColor = "Yellow",
}

a.FlashCDs = function()
	-- Actions.CD
	return a.Flash(
				"invoke_xuen_the_white_tiger"
				,"blood_fury"
				,"berserking"
				-- Use Arcane Torrent if you are missing at least 1 Chi and won't cap energy within 0.5 seconds
				,"arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5"
			)
--	,"lights_judgment"
	-- Cast ToD cycling through 2 targets if:
	-- - You're using HMFT
	-- - Your previous GCD was not ToD
--	,"touch_of_death,target_if=min:dot.touch_of_death.remains,if=equipped.hidden_masters_forbidden_touch&!prev_gcd.1.touch_of_death"
	-- The second cast of touch_of_death triggered by the legendary effect of hidden_masters_forbidden_touch:
	-- - You've already cast the first ToD
	-- - SEF is talented and will be available before your next Global Cooldown
	-- - Your previous GCD was not ToD
	-- - Remaining cooldown on Fist of Fury is lower or equal to 4 seconds
	-- - Remaining cooldown on Rising Sun Kick is lower than 7 seconds
--	,"touch_of_death,target_if=min:dot.touch_of_death.remains,if=((talent.serenity.enabled&cooldown.serenity.remains<=1)&cooldown.fists_of_fury.remains<=4)&cooldown.rising_sun_kick.remains<7&!prev_gcd.1.touch_of_death"
	-- The second cast of touch_of_death triggered by the legendary effect of hidden_masters_forbidden_touch:
	-- - You've already cast the first ToD
	-- - Remaining cooldown on Fists of Fury is lower or equal to 4 seconds AND SEF is talented and will be available before your next Global Cooldown OR you have 2 or more Chi
	-- - Your previous GCD was not ToD
	-- - Remaining cooldown on Rising Sun Kick is greather than 7 seconds
--	,"touch_of_death,target_if=min:dot.touch_of_death.remains,if=((!talent.serenity.enabled&cooldown.storm_earth_and_fire.remains<=1)|chi>=2)&cooldown.fists_of_fury.remains<=4&cooldown.rising_sun_kick.remains<7&!prev_gcd.1.touch_of_death"
end

--[[
     _         _____ 
    / \   ___ | ____|
   / _ \ / _ \|  _|  
  / ___ \ (_) | |___ 
 /_/   \_\___/|_____|
                     ]]

a.spells["energizing_elixir,if=!prev_gcd.1.tiger_palm&chi<=1&(cooldown.rising_sun_kick.remains=0|(talent.fist_of_the_white_tiger.enabled&cooldown.fist_of_the_white_tiger.remains=0)|energy<50)"] = {
	ID = 115288--[[Energizing Elixir]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm and x.Chi <= 1 and (cooldown.rising_sun_kick.remains == 0 or (talent.fist_of_the_white_tiger.enabled and cooldown.fist_of_the_white_tiger.remains == 0) or x.Energy < 50)
	end,
}

a.spells["fists_of_fury,if=talent.serenity.enabled&!equipped.drinking_horn_cover&cooldown.serenity.remains>=5&energy.time_to_max>2"] = {
	ID = 113656--[[Fists of Fury]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return talent.serenity.enabled and not equipped.drinking_horn_cover and cooldown.serenity.remains >= 5 and x.EnergyTimeToMax > 2
	end,
}

a.spells["fists_of_fury,if=talent.serenity.enabled&equipped.drinking_horn_cover&(cooldown.serenity.remains>=15|cooldown.serenity.remains<=4)&energy.time_to_max>2"] = {
	ID = 113656--[[Fists of Fury]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return talent.serenity.enabled and equipped.drinking_horn_cover and (cooldown.serenity.remains >= 15 or cooldown.serenity.remains <= 4) and x.EnergyTimeToMax > 2
	end,
}

a.spells["fists_of_fury,if=!talent.serenity.enabled&energy.time_to_max>2"] = {
	ID = 113656--[[Fists of Fury]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return not talent.serenity.enabled and x.EnergyTimeToMax > 2
	end,
}

a.spells["fists_of_fury,if=cooldown.rising_sun_kick.remains>=3.5&chi<=5"] = {
	ID = 113656--[[Fists of Fury]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return cooldown.rising_sun_kick.remains >= 3.5 and x.Chi <= 5
	end,
}

a.spells["whirling_dragon_punch"] = {
	ID = 152175--[[Whirling Dragon Punch]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return talent.whirling_dragon_punch.enabled
	end,
}

a.spells["rising_sun_kick,target_if=cooldown.whirling_dragon_punch.remains>=gcd&!prev_gcd.1.rising_sun_kick&cooldown.fists_of_fury.remains>gcd"] = {
	ID = 107428--[[Rising Sun Kick]],
	Check = function(z)
		return cooldown.whirling_dragon_punch.remains >= x.MaxGCD and not prev_gcd[1].rising_sun_kick and cooldown.fists_of_fury.remains > x.MaxGCD
	end,
}

a.spells["chi_burst,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1"] = {
	ID = 123986--[[Chi Burst]],
	NoRangeCheck = true,
	Check = function(z)
		return x.Chi <= 3 and (cooldown.rising_sun_kick.remains >= 5 or cooldown.whirling_dragon_punch.remains >= 5) and x.EnergyTimeToMax > 1
	end,
}

a.spells["chi_burst"] = {
	ID = 123986--[[Chi Burst]],
	NoRangeCheck = true,
}

a.spells["spinning_crane_kick,if=(active_enemies>=3|(buff.bok_proc.up&chi.max-chi>=0))&!prev_gcd.1.spinning_crane_kick&set_bonus.tier21_4pc"] = {
	ID = 101546--[[Spinning Crane Kick]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return (x.ActiveEnemies >= 3 or (buff.bok_proc.up and x.MaxChi - x.Chi >= 0)) and not prev_gcd[1].spinning_crane_kick and set_bonus.tier21_4pc
	end,
}

a.spells["spinning_crane_kick,if=active_enemies>=3&!prev_gcd.1.spinning_crane_kick"] = {
	ID = 101546--[[Spinning Crane Kick]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return x.ActiveEnemies >= 3 and not prev_gcd[1].spinning_crane_kick
	end,
}

a.spells["blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1&set_bonus.tier21_4pc&(!set_bonus.tier19_2pc|talent.serenity.enabled)"] = {
	ID = 100784--[[Blackout Kick]],
	Check = function(z)
		return not prev_gcd[1].blackout_kick and x.MaxChi - x.Chi >= 1 and set_bonus.tier21_4pc and (not set_bonus.tier19_2pc or talent.serenity.enabled)
	end,
}

a.spells["blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(chi>1|buff.bok_proc.up|(talent.energizing_elixir.enabled&cooldown.energizing_elixir.remains<cooldown.fists_of_fury.remains))&((cooldown.rising_sun_kick.remains>1&(!talent.fist_of_the_white_tiger.enabled|cooldown.fist_of_the_white_tiger.remains>1)|chi>4)&(cooldown.fists_of_fury.remains>1|chi>2)|prev_gcd.1.tiger_palm)&!prev_gcd.1.blackout_kick"] = {
	ID = 100784--[[Blackout Kick]],
	Check = function(z)
		return (x.Chi > 1 or buff.bok_proc.up or (talent.energizing_elixir.enabled and cooldown.energizing_elixir.remains < cooldown.fists_of_fury.remains)) and ((cooldown.rising_sun_kick.remains > 1 and (not talent.fist_of_the_white_tiger.enabled or cooldown.fist_of_the_white_tiger.remains > 1) or x.Chi > 4) and (cooldown.fists_of_fury.remains > 1 or x.Chi > 2) or prev_gcd[1].tiger_palm) and not prev_gcd[1].blackout_kick
	end,
}

a.spells["crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=19&energy.time_to_max>3"] = {
	ID = 117952--[[Crackling Jade Lightning]],
	Check = function(z)
		return equipped.the_emperors_capacitor and buff.the_emperors_capacitor.stack >= 19 and x.EnergyTimeToMax > 3
	end,
}

a.spells["crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=14&cooldown.serenity.remains<13&talent.serenity.enabled&energy.time_to_max>3"] = {
	ID = 117952--[[Crackling Jade Lightning]],
	Check = function(z)
		return equipped.the_emperors_capacitor and buff.the_emperors_capacitor.stack >= 14 and cooldown.serenity.remains < 13 and talent.serenity.enabled and x.EnergyTimeToMax > 3
	end,
}

a.spells["blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1&set_bonus.tier21_4pc&buff.bok_proc.up"] = {
	ID = 100784--[[Blackout Kick]],
	Check = function(z)
		return not prev_gcd[1].blackout_kick and x.MaxChi - x.Chi >= 1 and set_bonus.tier21_4pc and buff.bok_proc.up
	end,
}

a.spells["tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm and not prev_gcd[1].energizing_elixir and (x.MaxChi - x.Chi >= 2 or x.EnergyTimeToMax < 3)
	end,
}

a.spells["tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm and not prev_gcd[1].energizing_elixir and x.EnergyTimeToMax <= 1 and x.MaxChi - x.Chi >= 2
	end,
}

a.spells["chi_wave,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1"] = {
	ID = 115098--[[Chi Wave]],
	Check = function(z)
		return x.Chi <= 3 and (cooldown.rising_sun_kick.remains >= 5 or cooldown.whirling_dragon_punch.remains >= 5) and x.EnergyTimeToMax > 1
	end,
}

a.FlashAoE = function()
	-- Actions.AoE is intended for use with Hectic_Add_Cleave and currently needs to be optimized
	a.FlashCDs()
	s.UpdateDebuff(228287, "Mark of the Crane")
	s.UpdateBuff(116768, "BoK Proc")
	s.UpdateCooldown(115288, "Energizing Elixir")
	s.UpdateCooldown(152175, "Whirling Dragon Punch")
	s.UpdateCooldown(261947, "Fist of the White Tiger")
	return a.Flash(
		"energizing_elixir,if=!prev_gcd.1.tiger_palm&chi<=1&(cooldown.rising_sun_kick.remains=0|(talent.fist_of_the_white_tiger.enabled&cooldown.fist_of_the_white_tiger.remains=0)|energy<50)"
		,"fists_of_fury,if=talent.serenity.enabled&!equipped.drinking_horn_cover&cooldown.serenity.remains>=5&energy.time_to_max>2"
		,"fists_of_fury,if=talent.serenity.enabled&equipped.drinking_horn_cover&(cooldown.serenity.remains>=15|cooldown.serenity.remains<=4)&energy.time_to_max>2"
		,"fists_of_fury,if=!talent.serenity.enabled&energy.time_to_max>2"
		,"fists_of_fury,if=cooldown.rising_sun_kick.remains>=3.5&chi<=5"
		,"whirling_dragon_punch"
		,"rising_sun_kick,target_if=cooldown.whirling_dragon_punch.remains>=gcd&!prev_gcd.1.rising_sun_kick&cooldown.fists_of_fury.remains>gcd"
		,"chi_burst,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1"
		,"chi_burst"
		,"spinning_crane_kick,if=(active_enemies>=3|(buff.bok_proc.up&chi.max-chi>=0))&!prev_gcd.1.spinning_crane_kick&set_bonus.tier21_4pc"
		,"spinning_crane_kick,if=active_enemies>=3&!prev_gcd.1.spinning_crane_kick"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1&set_bonus.tier21_4pc&(!set_bonus.tier19_2pc|talent.serenity.enabled)"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(chi>1|buff.bok_proc.up|(talent.energizing_elixir.enabled&cooldown.energizing_elixir.remains<cooldown.fists_of_fury.remains))&((cooldown.rising_sun_kick.remains>1&(!talent.fist_of_the_white_tiger.enabled|cooldown.fist_of_the_white_tiger.remains>1)|chi>4)&(cooldown.fists_of_fury.remains>1|chi>2)|prev_gcd.1.tiger_palm)&!prev_gcd.1.blackout_kick"
		,"crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=19&energy.time_to_max>3"
		,"crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=14&cooldown.serenity.remains<13&talent.serenity.enabled&energy.time_to_max>3"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1&set_bonus.tier21_4pc&buff.bok_proc.up"
		,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)"
		,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2"
		,"chi_wave,if=chi<=3&(cooldown.rising_sun_kick.remains>=5|cooldown.whirling_dragon_punch.remains>=5)&energy.time_to_max>1"
		,"chi_wave"
	)
end

--[[
  ____  _                           _____           _   _                       _   _____ _          
 / ___|| |_ ___  _ __ _ __ ___     | ____|__ _ _ __| |_| |__     __ _ _ __   __| | |  ___(_)_ __ ___ 
 \___ \| __/ _ \| '__| '_ ` _ \    |  _| / _` | '__| __| '_ \   / _` | '_ \ / _` | | |_  | | '__/ _ \
  ___) | || (_) | |  | | | | | |_  | |__| (_| | |  | |_| | | | | (_| | | | | (_| | |  _| | | | |  __/
 |____/ \__\___/|_|  |_| |_| |_( ) |_____\__,_|_|   \__|_| |_|  \__,_|_| |_|\__,_| |_|   |_|_|  \___|
                               |/                                                                    ]]

a.spells["tiger_palm,target_if=debuff.mark_of_the_crane.down,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return debuff.mark_of_the_crane.down and not prev_gcd[1].tiger_palm and not prev_gcd[1].energizing_elixir and x.Energy == x.MaxEnergy and x.Chi < 1
	end,
}

a.spells["storm_earth_and_fire,if=!buff.storm_earth_and_fire.up"] = {
	ID = 137639--[[Storm, Earth, and Fire]],
	Check = function(z)
		return x.ActiveEnemy and x.UseLongCD and not buff.storm_earth_and_fire.up
	end,
}

a.FlashSEF = function()
	-- Actions.SEF
	s.UpdateDebuff(228287, "Mark of the Crane")
	if a.Flash("tiger_palm,target_if=debuff.mark_of_the_crane.down,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1") then return true end
	a.FlashCDs()
	if a.Flash("storm_earth_and_fire,if=!buff.storm_earth_and_fire.up") then return true end
	if x.ActiveEnemies > 3 then
		return a.FlashAoE()
	else
		return a.FlashSingleTarget()
	end
end
--[[
  ____                      _ _         
 / ___|  ___ _ __ ___ _ __ (_) |_ _   _ 
 \___ \ / _ \ '__/ _ \ '_ \| | __| | | |
  ___) |  __/ | |  __/ | | | | |_| |_| |
 |____/ \___|_|  \___|_| |_|_|\__|\__, |
                                  |___/ ]]
,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1&!buff.serenity.up"
,"serenity"
,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies<3"
,"fists_of_fury,if=((equipped.drinking_horn_cover&buff.pressure_point.remains<=2&set_bonus.tier20_4pc)&(cooldown.rising_sun_kick.remains>1|active_enemies>1)),interrupt=1"
,"fists_of_fury,if=((!equipped.drinking_horn_cover|buff.bloodlust.up|buff.serenity.remains<1)&(cooldown.rising_sun_kick.remains>1|active_enemies>1)),interrupt=1"
,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(!prev_gcd.1.blackout_kick)&(prev_gcd.1.fist_of_the_white_tiger|prev_gcd.1.fists_of_fury)&active_enemies<2"
,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies>=3"
,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick"

a.FlashSerenity = function()
	s.UpdateDebuff(228287, "Mark of the Crane")
	if a.Flash("tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1&!buff.serenity.up") then return true end
	a.FlashCDs()
	return a.Flash(
		"serenity"
		,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies<3"
		-- Legacy syntax for T19/T20 6pc
		,"fists_of_fury,if=((equipped.drinking_horn_cover&buff.pressure_point.remains<=2&set_bonus.tier20_4pc)&(cooldown.rising_sun_kick.remains>1|active_enemies>1)),interrupt=1"
		-- Cast Fist of Fury if:
		-- - The remaining cooldown on rising_sun_kick is longer than the channel duration of Fists_of_fury
		,"fists_of_fury,if=((!equipped.drinking_horn_cover|buff.bloodlust.up|buff.serenity.remains<1)&(cooldown.rising_sun_kick.remains>1|active_enemies>1)),interrupt=1"
		,"spinning_crane_kick,if=active_enemies>=3&!prev_gcd.1.spinning_crane_kick"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(!prev_gcd.1.blackout_kick)&(prev_gcd.1.fist_of_the_white_tiger|prev_gcd.1.fists_of_fury)&active_enemies<2"
		,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies>=3"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick"
	)
end

--[[
  ____                      _ _            ___                             
 / ___|  ___ _ __ ___ _ __ (_) |_ _   _   / _ \ _ __   ___ _ __   ___ _ __ 
 \___ \ / _ \ '__/ _ \ '_ \| | __| | | | | | | | '_ \ / _ \ '_ \ / _ \ '__|
  ___) |  __/ | |  __/ | | | | |_| |_| | | |_| | |_) |  __/ | | |  __/ |   
 |____/ \___|_|  \___|_| |_|_|\__|\__, |  \___/| .__/ \___|_| |_|\___|_|   
                                  |___/        |_|                         ]]
,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1&!buff.serenity.up&cooldown.fists_of_fury.remains<=0"
,"serenity,if=cooldown.fists_of_fury.remains>1"
,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies<3&buff.serenity.up"
,"fist_of_the_white_tiger,if=buff.serenity.up"
,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(!prev_gcd.1.blackout_kick)&(prev_gcd.1.fist_of_the_white_tiger)"
,"fists_of_fury,if=cooldown.rising_sun_kick.remains>1|buff.serenity.down,interrupt=1"
,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=buff.serenity.down&chi<=2&cooldown.serenity.remains<=0&prev_gcd.1.tiger_palm"
,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&chi=1"

a.FlashSerenityOpener = function()
	-- Actions.Serenity_Opener is Not Yet Implemented (NYI)
	s.UpdateDebuff(228287, "Mark of the Crane")
	if a.Flash("tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy=energy.max&chi<1&!buff.serenity.up&cooldown.fists_of_fury.remains<=0") then return true end
	a.Flash("arcane_torrent,if=chi.max-chi>=1&energy.time_to_max>=0.5")
	a.FlashCDs()
	return a.Flash(
		"serenity,if=cooldown.fists_of_fury.remains>1"
		,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=active_enemies<3&buff.serenity.up"
		-- Cast Fists_of_fury if
		-- - Rising Sun Kicks remaining cooldown is longer than 1 second
		-- - Interrupt Fists_of_fury with Rising Sun Kick if Serenity remains
		,"fist_of_the_white_tiger,if=buff.serenity.up"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=(!prev_gcd.1.blackout_kick)&(prev_gcd.1.fist_of_the_white_tiger)"
		,"fists_of_fury,if=cooldown.rising_sun_kick.remains>1|buff.serenity.down,interrupt=1"
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=buff.serenity.down&chi<=2&cooldown.serenity.remains<=0&prev_gcd.1.tiger_palm"
		,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&chi=1"		
	)
end

--[[
  ____  _             _        _____                    _   
 / ___|(_)_ __   __ _| | ___  |_   _|_ _ _ __ __ _  ___| |_ 
 \___ \| | '_ \ / _` | |/ _ \   | |/ _` | '__/ _` |/ _ \ __|
  ___) | | | | | (_| | |  __/   | | (_| | | | (_| |  __/ |_ 
 |____/|_|_| |_|\__, |_|\___|   |_|\__,_|_|  \__, |\___|\__|
                |___/                        |___/          ]]

a.spells["rushing_jade_wind,if=buff.rushing_jade_wind.down&!prev_gcd.1.rushing_jade_wind"] = {
	ID = 261715--[[Rushing Jade Wind]],
	Check = function(z)
		return x.InCombat and buff.rushing_jade_wind.down and not prev_gcd[1].rushing_jade_wind
	end,
	NoRangeCheck = true,
--	NoPowerCheck = true,
	Melee = true,
}

a.spells["energizing_elixir,if=!prev_gcd.1.tiger_palm"] = {
	ID = 115288--[[Energizing Elixir]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm
	end,
}

a.spells["tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2&!buff.serenity.up"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm and not prev_gcd[1].energizing_elixir and x.EnergyTimeToMax <= 1 and x.MaxChi - x.Chi >= 2 and not buff.serenity.up
	end,
}

a.spells["fist_of_the_white_tiger,if=chi.max-chi>=3"] = {
	ID = 261947--[[Fist of the White Tiger]],
	Check = function(z)
		return x.MaxChi - x.Chi >= 3
	end,
}

a.spells["rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=((chi>=3&energy>=40)|chi>=5)&(talent.serenity.enabled|cooldown.serenity.remains>=6)"] = {
	ID = 107428--[[Rising Sun Kick]],
	Check = function(z)
		return ((x.Chi >= 3 and x.Energy >= 40) or x.Chi >= 5) and (talent.serenity.enabled or cooldown.serenity.remains >= 6)
	end,
}

a.spells["fists_of_fury,if=!talent.serenity.enabled"] = {
	ID = 113656--[[Fists of Fury]],
	NoRangeCheck = true,
	Melee = true,
	Check = function(z)
		return not talent.serenity.enabled
	end,
}

a.spells["rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=cooldown.serenity.remains>=5|(!talent.serenity.enabled)"] = {
	ID = 107428--[[Rising Sun Kick]],
	Check = function(z)
		return cooldown.serenity.remains >= 5 or not talent.serenity.enabled
	end,
}

a.spells["blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1"] = {
	ID = 100784--[[Blackout Kick]],
	Check = function(z)
		return not prev_gcd[1].blackout_kick and x.MaxChi - x.Chi >= 1
	end,
}

a.spells["blackout_kick"] = {
	ID = 100784--[[Blackout Kick]],
	Check = function(z)
		return true
	end,
}

a.spells["chi_burst,if=energy.time_to_max>1&talent.serenity.enabled"] = {
	ID = 123986--[[Chi Burst]],
	NoRangeCheck = true,
	Check = function(z)
		return x.EnergyTimeToMax > 1 and talent.serenity.enabled
	end,
}

a.spells["tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)&!buff.serenity.up"] = {
	ID = 100780--[[Tiger Palm]],
	Check = function(z)
		return not prev_gcd[1].tiger_palm and not prev_gcd[1].energizing_elixir and (x.MaxChi - x.Chi >= 2 or x.EnergyTimeToMax < 3) and not buff.serenity.up
	end,
}

a.spells["chi_burst,if=chi.max-chi>=3&energy.time_to_max>1&!talent.serenity.enabled"] = {
	ID = 123986--[[Chi Burst]],
	NoRangeCheck = true,
	Check = function(z)
		return x.MaxChi - x.Chi >= 3 and x.EnergyTimeToMax > 1 and not talent.serenity.enabled
	end,
}

a.FlashSingleTarget = function()
	-- Default action list
	a.FlashCDs()
	s.UpdateDebuff(228287, "Mark of the Crane")
	s.UpdateBuff(261715, "Rushing Jade Wind")
	return a.Flash(
		"storm_earth_and_fire,if=!buff.storm_earth_and_fire.up"
		-- Needs to be rewritten for BFA
		,"rushing_jade_wind,if=buff.rushing_jade_wind.down&!prev_gcd.1.rushing_jade_wind"
		,"energizing_elixir,if=!prev_gcd.1.tiger_palm"
		-- T21 set bonus conditional
		-- Cast Blackout Kick if:
		-- - Previous GCD was not Blackout Kick
		-- - Blackout Kick! is available
		-- - You're not at max Chi
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1&set_bonus.tier21_4pc&buff.bok_proc.up"
		-- Cast Tiger Palm if:
		-- - Previous GCD was not Tiger Palm
		-- - Previous GCD was not EE (NOTE: redundant because of the Energy check, needs to be rewritten for BFA)
		-- - You will cap Energy before next GCD
		-- - You will gain 2 or more Chi
		,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&energy.time_to_max<=1&chi.max-chi>=2&!buff.serenity.up"
		-- Cast FotWT if you will gain 3 or more Chi
		,"fist_of_the_white_tiger,if=chi.max-chi>=3"
		,"whirling_dragon_punch"
		-- Cast Rising Sun Kick if:
		-- - You are using SEF, and you have 3 or more Chi AND 40 or more energy OR 5 or more Chi
		-- - You are using Serenity, 6 or more seconds remain on the cooldown of Serenity, and you have 3 or more Chi AND 40 or more energy OR 5 or more Chi 
		,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=((chi>=3&energy>=40)|chi>=5)&(talent.serenity.enabled|cooldown.serenity.remains>=6)"
		-- Legacy conditional for Drinking Horn Cover
		,"fists_of_fury,if=talent.serenity.enabled&!equipped.drinking_horn_cover&cooldown.serenity.remains>=5&energy.time_to_max>2"
		-- Legacy conditional for Drinking Horn Cover
		,"fists_of_fury,if=talent.serenity.enabled&equipped.drinking_horn_cover&(cooldown.serenity.remains>=15|cooldown.serenity.remains<=4)&energy.time_to_max>2"
		-- Cast Fists of Fury if:
		-- - You are using SEF
		,"fists_of_fury,if=!talent.serenity.enabled"
		-- Cast RSK if:
		-- - You are using SEF OR you are using Serenity and 5 or more seconds remain on the cooldown of Serenity
		,"rising_sun_kick,target_if=min:debuff.mark_of_the_crane.remains,if=cooldown.serenity.remains>=5|(!talent.serenity.enabled)"
		-- Cast Blackout Kick if:
		-- - Previous GCD was not BoK
		-- - You are not at max Chi
		,"blackout_kick,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.blackout_kick&chi.max-chi>=1"
		-- Legacy for The Emperors Capacitor
		,"crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=19&energy.time_to_max>3"
		,"crackling_jade_lightning,if=equipped.the_emperors_capacitor&buff.the_emperors_capacitor.stack>=14&cooldown.serenity.remains<13&talent.serenity.enabled&energy.time_to_max>3"
--		,"blackout_kick"
		,"chi_wave"
		-- Will need to be rewritten for BFA
		-- Current rule: Cast Chi Burst if:
		-- - You have 3 or less Chi
		-- - RSK is up in 5 or more seconds OR WDP is up in 5 or more seconds
		-- - You will not cap energy before the next GCD
		,"chi_burst,if=energy.time_to_max>1&talent.serenity.enabled"
		-- Cast Tiger Palm if:
		-- - Previous ability was not Tiger Palm or Energizing Elixir
		-- - You will gain at least 2 Chi, OR you will cap energy within 3 seconds (NOTE: Could cast TP even at 5 Chi just to prevent energy capping, in theory)
		,"tiger_palm,target_if=min:debuff.mark_of_the_crane.remains,if=!prev_gcd.1.tiger_palm&!prev_gcd.1.energizing_elixir&(chi.max-chi>=2|energy.time_to_max<3)&!buff.serenity.up"
		,"chi_burst,if=chi.max-chi>=3&energy.time_to_max>1&!talent.serenity.enabled"
	)
end
